@@ -1,6 +1,6 @@
 package ExtUtils::Constant;
 use vars qw (@ISA $VERSION @EXPORT_OK %EXPORT_TAGS);
-$VERSION = '0.23_05';
+$VERSION = '0.23_06';
 $VERSION = eval $VERSION;
 
 =head1 NAME
@@ -25,6 +25,9 @@ It is principally used by the C<h2xs> utility, on which this code is based.
 It doesn't contain the routines to scan header files to extract these
 constants.
 
+Memory footprint and run-time performance is not as good as
+specialized perfect hashes as with L<XSConfig> or L<Win32::GUI::Constants>.
+
 =head1 USAGE
 
 Generally one only needs to call the C<WriteConstants> function, and then
@@ -91,8 +94,10 @@ C<undef>.  The value of the macro is not needed.
 
 =cut
 
-if ($] >= 5.006) {
-  eval "use warnings; 1" or die $@;
+BEGIN {
+    if ($] >= 5.006) {
+        eval "use warnings; 1" or die $@;
+    }
 }
 use strict;
 use Carp qw(croak cluck);
@@ -261,6 +266,8 @@ EOT
   # If anyone is insane enough to suggest a package name containing %
   my $package_sprintf_safe = $package;
   $package_sprintf_safe =~ s/%/%%/g;
+  # People were actually more insane than thought
+  $package_sprintf_safe =~ s/\x{0}/\\0/g if $] > 5.015006;
 
   $xs .= << "EOT";
       /* Return 1 or 2 items. First is error message, or undef if no error.
@@ -322,6 +329,9 @@ It defaults to the version of perl running the subroutine.  If I<AUTOLOADER>
 is true, the AUTOLOAD subroutine falls back on AutoLoader::AUTOLOAD for all
 names that the constant() routine doesn't recognise.
 
+This is needed unless you use C<PROXYSUBS => {autoload=>1}>, but which generates
+code unusable earlier than 5.8.
+
 =cut
 
 # ' # Grr. syntax highlighters that don't grok pod.
@@ -404,24 +414,26 @@ sub WriteMakefileSnippet {
   my $indent = $args{INDENT} || 2;
 
   my $result = <<"EOT";
-ExtUtils::Constant::WriteConstants(
-                                   NAME         => '$args{NAME}',
-                                   NAMES        => \\\@names,
-                                   DEFAULT_TYPE => '$args{DEFAULT_TYPE}',
+ExtUtils::Constant::WriteConstants
+    (
+      NAME         => '$args{NAME}',
+      NAMES        => \\\@names,
+      DEFAULT_TYPE => '$args{DEFAULT_TYPE}',
 EOT
   foreach (qw (C_FILE XS_FILE)) {
     next unless exists $args{$_};
-    $result .= sprintf "                                   %-12s => '%s',\n",
-      $_, $args{$_};
+    $result .= sprintf "      %-12s => '%s',\n",
+                 $_, $args{$_};
   }
   $result .= <<'EOT';
-                                );
+    );
 EOT
 
   $result =~ s/^/' 'x$indent/gem;
-  return ExtUtils::Constant::XS->dump_names({default_type=>$args{DEFAULT_TYPE},
-					     indent=>$indent,},
-					    @{$args{NAMES}})
+  return ExtUtils::Constant::XS->dump_names
+    ({default_type=>$args{DEFAULT_TYPE},
+      indent=>$indent,},
+     @{$args{NAMES}})
     . $result;
 }
 
@@ -435,47 +447,40 @@ The attributes supported are
 
 =over 4
 
-=item NAME
+=item C<NAME>
 
 Name of the module.  This must be specified
 
-=item DEFAULT_TYPE
+=item C<DEFAULT_TYPE>
 
 The default type for the constants.  If not specified C<IV> is assumed.
 
-=item BREAKOUT_AT
+=item C<BREAKOUT_AT>
 
 The names of the constants are grouped by length.  Generate child subroutines
 for each group with this number or more names in.
 
-=item NAMES
+=item C<NAMES>
 
 An array of constants' names, either scalars containing names, or hashrefs
 as detailed in L<"C_constant">.
 
-=item PROXYSUBS
+=item C<PROXYSUBS>
 
 If true, uses proxy subs. See L<ExtUtils::Constant::ProxySubs>.
-Options: autoload, push, croak_on_error or croak_on_read with the
-options being exclusive.
+PROXYSUBS create CONSTSUB's for each defined constant upfront, while
+without PROXYSUBS every constant is looked up at run-time. Thus it
+trades memory footprint for faster run-time performance.
 
-Warning: Code generated via ProxySubs with any of the options push,
-croak_on_error or croak_on_read will only work for newer Perl versions
-and thus should not be used for CPAN modules.
-push can only be used with >= 5.10,
-croak_on_error >= 5.14,
-croak_on_read >= 5.24,
-ProxySubs or with autoload >= 5.8.
+Options: autoload, push, croak_on_error or croak_on_read with most of
+the options being exclusive, and croak_on_read usable since 5.24.
 
-ExtUtils::Constant::ProxySubs versions older than 0.23_04 even creates
-code usable only >=5.14.
-
-=item C_FH
+=item C<C_FH>
 
 A filehandle to write the C code to.  If not given, then I<C_FILE> is opened
 for writing.
 
-=item C_FILE
+=item C<C_FILE>
 
 The name of the file to write containing the C code.  The default is
 C<const-c.inc>.  The C<-> in the name ensures that the file can't be
@@ -483,22 +488,22 @@ mistaken for anything related to a legitimate perl package name, and
 not naming the file C<.c> avoids having to override Makefile.PL's
 C<.xs> to C<.c> rules.
 
-=item XS_FH
+=item C<XS_FH>
 
 A filehandle to write the XS code to.  If not given, then I<XS_FILE> is opened
 for writing.
 
-=item XS_FILE
+=item C<XS_FILE>
 
 The name of the file to write containing the XS code.  The default is
 C<const-xs.inc>.
 
-=item XS_SUBNAME
+=item C<XS_SUBNAME>
 
 The perl visible name of the XS subroutine generated which will return the
 constants. The default is C<constant>.
 
-=item C_SUBNAME
+=item C<C_SUBNAME>
 
 The name of the C subroutine generated which will return the constants.
 The default is I<XS_SUBNAME>.  Child subroutines have C<_> and the name
@@ -536,7 +541,7 @@ sub WriteConstants {
 	  require FileHandle;
 	  $c_fh = FileHandle->new();
       }
-      open $c_fh, ">$ARGS{C_FILE}" or die "Can't open $ARGS{C_FILE}: $!";
+      open $c_fh, ">", $ARGS{C_FILE} or die "Can't open $ARGS{C_FILE}: $!";
   }
 
   my $xs_fh = $ARGS{XS_FH};
@@ -545,7 +550,7 @@ sub WriteConstants {
 	  require FileHandle;
 	  $xs_fh = FileHandle->new();
       }
-      open $xs_fh, ">$ARGS{XS_FILE}" or die "Can't open $ARGS{XS_FILE}: $!";
+      open $xs_fh, ">", $ARGS{XS_FILE} or die "Can't open $ARGS{XS_FILE}: $!";
   }
 
   # As this subroutine is intended to make code that isn't edited, there's no
@@ -564,14 +569,16 @@ sub WriteConstants {
 
       # indent is still undef. Until anyone implements indent style rules with
       # it.
-      foreach (ExtUtils::Constant::XS->C_constant({package => $ARGS{NAME},
-						   subname => $ARGS{C_SUBNAME},
-						   default_type =>
-						       $ARGS{DEFAULT_TYPE},
-						       types => $types,
-						       breakout =>
-						       $ARGS{BREAKOUT_AT}},
-						  @{$ARGS{NAMES}})) {
+      foreach (ExtUtils::Constant::XS->C_constant
+               ({package => $ARGS{NAME},
+                 subname => $ARGS{C_SUBNAME},
+                 default_type =>
+                   $ARGS{DEFAULT_TYPE},
+                 types => $types,
+                 breakout =>
+                   $ARGS{BREAKOUT_AT}},
+                @{$ARGS{NAMES}}))
+      {
 	  print $c_fh $_, "\n"; # C constant subs
       }
       print $xs_fh XS_constant ($ARGS{NAME}, $types, $ARGS{XS_SUBNAME},
@@ -589,7 +596,8 @@ __END__
 
 =head1 AUTHOR
 
-Nicholas Clark <nick@ccl4.org> based on the code in C<h2xs> by Larry Wall and
-others
+Reini Urban <rurban@cpan.org> fixed up ProxySubs and took over maintainance.
+Nicholas Clark <nick@ccl4.org> wrote it based on the code in C<h2xs>
+by Larry Wall and others.
 
 =cut
